/*+------------------------------------------------------------------
  ply.cpp - ply file implementation file

  Clifford M. Bryant, Jr.
*/

#include "windows.h"
#include <fstream>
#include <stdio.h>
#include "ply.h"


double CVertex::min[3];
double CVertex::max[3];

CVertex * CTriangle::m_pVertices = NULL;
int CTriangle::m_iShading = GL_SMOOTH;

int CPly::read(const string strFilename)
{
	ifstream ifs;
	const int BUFF_SIZE = 256;
	char buff[BUFF_SIZE];
	string strBuffer;
	string str;

	// Open the ply file.
	ifs.open(strFilename.c_str());
	if (ifs.fail())
	{
		string strMsg = "Unable to open PLY file " + strFilename;
		::MessageBox(NULL, strFilename.c_str(), "Open Error", MB_OK);
		return -1;
	}

	// Read the ply file identifier
	ifs >> strBuffer;
	if (strBuffer != "ply")
	{
		string strMsg = "File " + strFilename + " is not a PLY file.";
		::MessageBox(NULL, strFilename.c_str(), "File Error", MB_OK);
		return -1;
	}

	// Skip the "format ascii 1.0" line.
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);
	
	// Read the number of vertices.
	ifs >> str >> str >> m_iVertices;

	// Skip over the 3 "property" lines.
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);
	
	// Read the number of polygons (triangles).
	ifs >> str >> str >> m_iTriangles;

	// Skip over the single "property" line and
	// the end of the header indicator.
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);
	ifs.getline(buff, BUFF_SIZE);

	// Allocate memory for the vertices.
	if (m_pVertices) delete [] m_pVertices;
	m_pVertices = new CVertex[m_iVertices];

	// Read in the vertices.
	CVertex::reset();
	for (int i = 0; i < m_iVertices; i++)
		ifs >> m_pVertices[i];

	// Allocate memory for the triangles.
	if (m_pTriangles) delete [] m_pTriangles;
	m_pTriangles = new CTriangle[m_iTriangles];

	CTriangle::setVertices(m_pVertices);

	// Read in the triangles.
	for (i = 0; i < m_iTriangles; i++)
		ifs >> m_pTriangles[i];

	return 0;
}

void CVertex::calcStatistics()
{
	min[0] = __min(c[0], min[0]);
	min[1] = __min(c[1], min[1]);
	min[2] = __min(c[2], min[2]);
	max[0] = __max(c[0], max[0]);
	max[1] = __max(c[1], max[1]);
	max[2] = __max(c[2], max[2]);
}

void CVertex::sumNormal(double * _n)
{
	n[0] += _n[0];
	n[1] += _n[1];
	n[2] += _n[2];
}

void CVertex::normalize()
{
	// Normalize the normal vector.
	double d = sqrt(n[0] * n[0] + n[1] * n[1] + n[2] * n[2]);
	n[0] /= d;
	n[1] /= d;
	n[2] /= d;
}

istream & operator>>(istream & is, CVertex & v)
{
	is >> v.c[0] >> v.c[1] >> v.c[2];
	v.calcStatistics();
	return is;
}

void CTriangle::draw()
{
	if (m_iShading == GL_FLAT)
	{
		glBegin(GL_TRIANGLES);
			glNormal3dv(n);
			glVertex3dv(m_pVertices[v[0]].c);
			glVertex3dv(m_pVertices[v[1]].c);
			glVertex3dv(m_pVertices[v[2]].c);
		glEnd();
	}
	else
	{
		glBegin(GL_TRIANGLES);
			glNormal3dv(m_pVertices[v[0]].n);
			glVertex3dv(m_pVertices[v[0]].c);
			glNormal3dv(m_pVertices[v[1]].n);
			glVertex3dv(m_pVertices[v[1]].c);
			glNormal3dv(m_pVertices[v[2]].n);
			glVertex3dv(m_pVertices[v[2]].c);
		glEnd();
	}
}

void CTriangle::calcNormal()
{
	double v1[3], v2[3];

	// Calculate the two vectors from the three points.
	v1[0] = m_pVertices[v[1]].c[0] - m_pVertices[v[0]].c[0];
	v1[1] = m_pVertices[v[1]].c[1] - m_pVertices[v[0]].c[1];
	v1[2] = m_pVertices[v[1]].c[2] - m_pVertices[v[0]].c[2];
	v2[0] = m_pVertices[v[2]].c[0] - m_pVertices[v[1]].c[0];
	v2[1] = m_pVertices[v[2]].c[1] - m_pVertices[v[1]].c[1];
	v2[2] = m_pVertices[v[2]].c[2] - m_pVertices[v[1]].c[2];

	// Take the cross product to get the normal vector.
	n[0] = v1[1] * v2[2] - v2[1] * v1[2];
	n[1] = v1[2] * v2[0] - v2[2] * v1[0];
	n[2] = v1[0] * v2[1] - v2[0] * v1[1];

	// Normalize the normal vector.
	double d = sqrt(n[0] * n[0] + n[1] * n[1] + n[2] * n[2]);
	n[0] /= d;
	n[1] /= d;
	n[2] /= d;
}

void CTriangle::sumNormals()
{
	m_pVertices[v[0]].sumNormal(n);
	m_pVertices[v[1]].sumNormal(n);
	m_pVertices[v[2]].sumNormal(n);
}

istream & operator>>(istream & is, CTriangle & t)
{
	int nv;
	is >> nv >> t.v[0] >> t.v[1] >> t.v[2];
	t.calcNormal();
	return is;
}

void CPly::draw()
{
	for (int i = 0; i < m_iTriangles; i++)
		m_pTriangles[i].draw();
}

void CPly::calcNormals()
{
	for (int i = 0; i < m_iTriangles; i++)
		m_pTriangles[i].sumNormals();
	for (i = 0; i < m_iVertices; i++)
		m_pVertices[i].normalize();
}
